@@ -8,8 +8,8 @@ from cmor.cmor_const import *
 
 from cmor.pywrapper import (
     CMORError, axis, variable, write, setup, load_table, set_table, zfactor,
-    close, grid, set_grid_mapping, time_varying_grid_coordinate, dataset_json,
-    set_cur_dataset_attribute, get_cur_dataset_attribute,
+    close, grid, set_grid_mapping, set_crs, time_varying_grid_coordinate,
+    dataset_json, set_cur_dataset_attribute, get_cur_dataset_attribute,
     has_cur_dataset_attribute, set_variable_attribute, get_variable_attribute,
     has_variable_attribute, get_final_filename, set_deflate, set_zstandard,
     set_quantize, set_furtherinfourl, set_climatology, get_climatology,
