@@ -42,7 +42,6 @@ void cmor_init_grid_mapping(cmor_mappings_t * mapping, char *id)
 int cmor_copy_data(double **dest1, void *data, char type, int nelts)
 {
     int i;
-    char msg[CMOR_MAX_STRING];
     double *dest;
 
     cmor_add_traceback("cmor_copy_data");
@@ -64,8 +63,8 @@ int cmor_copy_data(double **dest1, void *data, char type, int nelts)
         else if (type == 'd')
             dest[i] = (double)((double *)data)[i];
         else {
-            snprintf(msg, CMOR_MAX_STRING, "wrong data type: %c", type);
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "wrong data type: %c", CMOR_CRITICAL, type);
         }
     }
     *dest1 = dest;
@@ -124,7 +123,6 @@ void cmor_convert_value(char *units, char *ctmp, double *tmp)
     ut_unit *user_units = NULL, *cmor_units = NULL;
     cv_converter *ut_cmor_converter = NULL;
     double value;
-    char msg[CMOR_MAX_STRING];
 
     cmor_add_traceback("cmor_convert_value");
 
@@ -134,26 +132,26 @@ void cmor_convert_value(char *units, char *ctmp, double *tmp)
                         &ut_cmor_converter);
         *tmp = cv_convert_double(ut_cmor_converter, value);
         if (ut_get_status() != UT_SUCCESS) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "Udunits: Error converting units from %s to %s",
-                     units, ctmp);
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "Udunits: Error converting units from %s to %s",
+                CMOR_CRITICAL,
+                units, ctmp);
         }
 
         cv_free(ut_cmor_converter);
         if (ut_get_status() != UT_SUCCESS) {
-            snprintf(msg, CMOR_MAX_STRING, "Udunits: Error freeing converter");
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "Udunits: Error freeing converter", CMOR_CRITICAL);
         }
         ut_free(cmor_units);
         if (ut_get_status() != UT_SUCCESS) {
-            snprintf(msg, CMOR_MAX_STRING, "Udunits: Error freeing units");
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "Udunits: Error freeing units", CMOR_CRITICAL);
         }
         ut_free(user_units);
         if (ut_get_status() != UT_SUCCESS) {
-            snprintf(msg, CMOR_MAX_STRING, "Udunits: Error freeing units");
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "Udunits: Error freeing units", CMOR_CRITICAL);
         }
     } else
         *tmp = value;
@@ -170,7 +168,6 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
     int i, j, iatt;
     int grid_id;
     char ctmp[CMOR_MAX_STRING];
-    char msg[CMOR_MAX_STRING];
     double tmp;
 
     cmor_add_traceback("cmor_set_grid_attribute");
@@ -203,9 +200,12 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
                 j = i;
         }
         if (j == -1) {
-            snprintf(ctmp, CMOR_MAX_STRING,
-                     "grid mapping attribute: 'false easting' must be set in conjunction with ut_cmor_a 'projection_x_coordinate' axis, I could not find such an axis on your grid, we will not set this attribute");
-            cmor_handle_error(ctmp, CMOR_NORMAL);
+            cmor_handle_error_variadic(
+                "grid mapping attribute: 'false easting' "
+                "must be set in conjunction with ut_cmor_a "
+                "'projection_x_coordinate' axis, I could not find "
+                "such an axis on your grid, we will not set this attribute",
+                CMOR_NORMAL);
             cmor_pop_traceback();
             return (1);
         }
@@ -221,9 +221,12 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
                 j = i;
         }
         if (j == -1) {
-            snprintf(ctmp, CMOR_MAX_STRING,
-                     "grid mapping attribute: 'false northing' must be set in conjunction with a 'projection_y_coordinate' axis, I could not find such an axis on your grid, we will not set this attribute");
-            cmor_handle_error(ctmp, CMOR_NORMAL);
+            cmor_handle_error_variadic(
+                "grid mapping attribute: 'false northing' "
+                "must be set in conjunction with a 'projection_y_coordinate' "
+                "axis, I could not find such an axis on your grid, "
+                "we will not set this attribute",
+                CMOR_NORMAL);
             cmor_pop_traceback();
             return (1);
         }
@@ -238,10 +241,10 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
         strcpy(ctmp, "degrees_north");
         cmor_convert_value(units, ctmp, &tmp);
         if ((tmp < -90) || (tmp > 90.)) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "%s parameter must be between -90 and 90 %s, will not be set",
-                     name, ctmp);
-            cmor_handle_error(msg, CMOR_NORMAL);
+            cmor_handle_error_variadic(
+                "%s parameter must be between -90 and 90 %s, will not be set",
+                CMOR_NORMAL,
+                name, ctmp);
             cmor_pop_traceback();
             return (1);
         }
@@ -249,15 +252,14 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
                || strcmp(name, "longitude_of_prime_meridian") == 0
                || strcmp(name, "longitude_of_central_meridian") == 0
                || strcmp(name, "longitude_of_projection_origin") == 0
-               || strcmp(name, "north_pole_grid_longitude") == 0
-               || strcmp(name, "straight_vertical_longitude_from_pole") == 0) {
+               || strcmp(name, "north_pole_grid_longitude") == 0) {
         strcpy(ctmp, "degrees_east");
         cmor_convert_value(units, ctmp, &tmp);
         if ((tmp < -180) || (tmp > 180.)) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "%s parameter must be between -180 and 180 %s, will not be set",
-                     name, ctmp);
-            cmor_handle_error(msg, CMOR_NORMAL);
+            cmor_handle_error_variadic(
+                "%s parameter must be between -180 and 180 %s, will not be set",
+                CMOR_NORMAL,
+                name, ctmp);
             cmor_pop_traceback();
             return (1);
         }
@@ -271,10 +273,10 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
         strcpy(ctmp, "m");
         cmor_convert_value(units, ctmp, &tmp);
         if (tmp < 0) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "%s parameter must be between positive, will not be set",
-                     name);
-            cmor_handle_error(msg, CMOR_NORMAL);
+            cmor_handle_error_variadic(
+                "%s parameter must be between positive, will not be set",
+                CMOR_NORMAL,
+                name);
             cmor_pop_traceback();
             return (1);
         }
@@ -286,6 +288,82 @@ int cmor_set_grid_attribute(int gid, char *name, double *value, char *units)
     return (0);
 }
 
+/************************************************************************/
+/*                    cmor_has_grid_text_attribute()                    */
+/************************************************************************/
+
+int cmor_has_grid_text_attribute(int gid, char *name)
+{
+    int i;
+    int grid_id;
+
+    grid_id = -gid - CMOR_MAX_GRIDS;
+    for (i = 0; i < cmor_grids[grid_id].ntextattributes; i++) {
+        if (strcmp(name, cmor_grids[grid_id].text_attributes_names[i]) == 0)
+            return (0);
+    }
+    return (1);
+}
+
+/************************************************************************/
+/*                    cmor_get_grid_text_attribute()                    */
+/************************************************************************/
+
+int cmor_get_grid_text_attribute(int gid, char *name, char *value)
+{
+    int i, j;
+    int grid_id;
+
+    grid_id = -gid - CMOR_MAX_GRIDS;
+    j = -1;
+    for (i = 0; i < cmor_grids[grid_id].ntextattributes; i++) {
+        if (strcmp(name, cmor_grids[grid_id].text_attributes_names[i]) == 0)
+            j = i;
+    }
+    if (j != -1) {
+        value = cmor_grids[grid_id].text_attributes_values[j];
+        return (0);
+    }
+    return (1);
+}
+
+/************************************************************************/
+/*                    cmor_set_grid_text_attribute()                    */
+/************************************************************************/
+
+int cmor_set_grid_text_attribute(int gid, char *name, char *value, int size)
+{
+    int i, iatt;
+    int grid_id;
+
+    cmor_add_traceback("cmor_set_grid_text_attribute");
+    grid_id = -gid - CMOR_MAX_GRIDS;
+    iatt = cmor_grids[grid_id].ntextattributes;
+
+    for (i = 0; i < cmor_grids[grid_id].ntextattributes; i++) {
+        if (strcmp(name, cmor_grids[grid_id].text_attributes_names[i]) == 0)
+            iatt = i;
+    }
+
+    if (iatt == cmor_grids[grid_id].ntextattributes)
+        cmor_grids[grid_id].ntextattributes++;
+
+    strncpy(cmor_grids[grid_id].text_attributes_names[iatt], name, CMOR_MAX_STRING);
+
+    if (cmor_grids[grid_id].text_attributes_values[iatt] != NULL) {
+        free(cmor_grids[grid_id].text_attributes_values[iatt]);
+        cmor_grids[grid_id].text_attributes_values[iatt] = NULL;
+    }
+
+    if (value != NULL) {
+        cmor_grids[grid_id].text_attributes_values[iatt] = malloc(size * sizeof(char));
+        strncpy(cmor_grids[grid_id].text_attributes_values[iatt], value, size);
+    }
+
+    cmor_pop_traceback();
+    return (0);
+}
+
 /************************************************************************/
 /*                       cmor_attribute_in_list()                       */
 /************************************************************************/
@@ -323,8 +401,8 @@ int cmor_grid_valid_mapping_attribute_names(char *name, int *natt, char (*att)
         strcpy(att[3], "false_easting");
         strcpy(att[4], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "azimuthal_equidistant") == 0) {
         *natt = 4;
         strcpy(att[0], "longitude_of_projection_origin");
@@ -332,17 +410,29 @@ int cmor_grid_valid_mapping_attribute_names(char *name, int *natt, char (*att)
         strcpy(att[2], "false_easting");
         strcpy(att[3], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
+    } else if (strcmp(name, "geostationary") == 0) {
+        *natt = 7;
+        strcpy(att[0], "latitude_of_projection_origin");
+        strcpy(att[1], "longitude_of_projection_origin");
+        strcpy(att[2], "perspective_point_height");
+        strcpy(att[3], "sweep_angle_axis");
+        strcpy(att[4], "fixed_angle_axis");
+        strcpy(att[5], "false_easting");
+        strcpy(att[6], "false_northing");
+        *ndims = 2;
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "lambert_azimuthal_equal_area") == 0) {
         *natt = 4;
-        strcpy(att[0], "longitude_of_projection_origin");
-        strcpy(att[1], "latitude_of_projection_origin");
+        strcpy(att[0], "latitude_of_projection_origin");
+        strcpy(att[1], "longitude_of_projection_origin");
         strcpy(att[2], "false_easting");
         strcpy(att[3], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "lambert_conformal_conic") == 0) {
         *natt = 5;
         strcpy(att[0], "standard_parallel");
@@ -351,52 +441,62 @@ int cmor_grid_valid_mapping_attribute_names(char *name, int *natt, char (*att)
         strcpy(att[3], "false_easting");
         strcpy(att[4], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "lambert_cylindrical_equal_area") == 0) {
-        *natt = 5;
+        *natt = 4;
         strcpy(att[0], "standard_parallel");
         strcpy(att[1], "longitude_of_central_meridian");
-        strcpy(att[2], "scale_factor_at_projection_origin");
-        strcpy(att[3], "false_easting");
-        strcpy(att[4], "false_northing");
+        strcpy(att[2], "false_easting");
+        strcpy(att[3], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "latitude_longitude") == 0) {
         *natt = 0;
-        *ndims = 0;
+        *ndims = 2;
+        strcpy(dims[0], "latitude");
+        strcpy(dims[1], "longitude");
     } else if (strcmp(name, "mercator") == 0) {
         *natt = 5;
-        strcpy(att[0], "standard_parallel");
-        strcpy(att[1], "longitude_of_projection_origin");
-        strcpy(att[2], "scale_factor_at_projection_origin");
-        strcpy(att[3], "false_easting");
-        strcpy(att[4], "false_northing");
-        *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
-    } else if (strcmp(name, "orthographic") == 0) {
-        *natt = 4;
         strcpy(att[0], "longitude_of_projection_origin");
-        strcpy(att[1], "latitude_of_projection_origin");
+        strcpy(att[1], "standard_parallel");
         strcpy(att[2], "scale_factor_at_projection_origin");
         strcpy(att[3], "false_easting");
         strcpy(att[4], "false_northing");
         *ndims = 2;
         strcpy(dims[0], "projection_x_coordinate");
         strcpy(dims[1], "projection_y_coordinate");
-    } else if (strcmp(name, "polar_stereographic") == 0) {
+    } else if (strcmp(name, "oblique_mercator") == 0) {
         *natt = 6;
-        strcpy(att[0], "straight_vertical_longitude_from_pole");
+        strcpy(att[0], "azimuth_of_central_line");
         strcpy(att[1], "latitude_of_projection_origin");
-        strcpy(att[2], "standard_parallel");
+        strcpy(att[2], "longitude_of_projection_origin");
         strcpy(att[3], "scale_factor_at_projection_origin");
         strcpy(att[4], "false_easting");
         strcpy(att[5], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
+    } else if (strcmp(name, "orthographic") == 0) {
+        *natt = 4;
+        strcpy(att[0], "latitude_of_projection_origin");
+        strcpy(att[1], "longitude_of_projection_origin");
+        strcpy(att[2], "false_easting");
+        strcpy(att[3], "false_northing");
+        *ndims = 2;
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
+    } else if (strcmp(name, "polar_stereographic") == 0) {
+        *natt = 5;
+        strcpy(att[0], "latitude_of_projection_origin");
+        strcpy(att[1], "longitude_of_projection_origin");
+        strcpy(att[2], "standard_parallel");
+        strcpy(att[3], "false_easting");
+        strcpy(att[4], "false_northing");
+        *ndims = 2;
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "rotated_latitude_longitude") == 0) {
         *natt = 3;
         strcpy(att[0], "grid_north_pole_latitude");
@@ -405,16 +505,24 @@ int cmor_grid_valid_mapping_attribute_names(char *name, int *natt, char (*att)
         *ndims = 2;
         strcpy(dims[0], "grid_latitude");
         strcpy(dims[1], "grid_longitude");
+    } else if (strcmp(name, "sinusoidal") == 0) {
+        *natt = 3;
+        strcpy(att[0], "longitude_of_projection_origin");
+        strcpy(att[1], "false_easting");
+        strcpy(att[2], "false_northing");
+        *ndims = 2;
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "stereographic") == 0) {
         *natt = 5;
-        strcpy(att[0], "longitude_of_projection_origin");
-        strcpy(att[1], "latitude_of_projection_origin");
+        strcpy(att[0], "latitude_of_projection_origin");
+        strcpy(att[1], "longitude_of_projection_origin");
         strcpy(att[2], "scale_factor_at_projection_origin");
         strcpy(att[3], "false_easting");
         strcpy(att[4], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "transverse_mercator") == 0) {
         *natt = 5;
         strcpy(att[0], "scale_factor_at_central_meridian");
@@ -423,18 +531,18 @@ int cmor_grid_valid_mapping_attribute_names(char *name, int *natt, char (*att)
         strcpy(att[3], "false_easting");
         strcpy(att[4], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     } else if (strcmp(name, "vertical_perspective") == 0) {
         *natt = 5;
-        strcpy(att[0], "longitude_of_projection_origin");
-        strcpy(att[1], "latitude_of_projection_origin");
-        strcpy(att[2], "perspective_height_point");
+        strcpy(att[0], "latitude_of_projection_origin");
+        strcpy(att[1], "longitude_of_projection_origin");
+        strcpy(att[2], "perspective_point_height");
         strcpy(att[3], "false_easting");
         strcpy(att[4], "false_northing");
         *ndims = 2;
-        strcpy(dims[0], "projection_y_coordinate");
-        strcpy(dims[1], "projection_x_coordinate");
+        strcpy(dims[0], "projection_x_coordinate");
+        strcpy(dims[1], "projection_y_coordinate");
     }
 
     /* Now looks up in table */
@@ -455,10 +563,11 @@ int cmor_grid_valid_mapping_attribute_names(char *name, int *natt, char (*att)
         strcpy(att[*natt + 0], "earth_radius");
         strcpy(att[*natt + 1], "inverse_flattening");
         strcpy(att[*natt + 2], "longitude_of_prime_meridian");
-        strcpy(att[*natt + 3], "perspective_point_height");
-        strcpy(att[*natt + 4], "semi_major_axis");
-        strcpy(att[*natt + 5], "semi_minor_axis");
-        *natt = *natt + 6;
+        strcpy(att[*natt + 3], "prime_meridian_name");
+        strcpy(att[*natt + 4], "reference_ellipsoid_name");
+        strcpy(att[*natt + 5], "semi_major_axis");
+        strcpy(att[*natt + 6], "semi_minor_axis");
+        *natt = *natt + 7;
     }
     return (0);
 }
@@ -468,7 +577,6 @@ void cmor_set_mapping_attribute(cmor_mappings_t * mapping,
                                 char val[CMOR_MAX_STRING])
 {
     int i, n;
-    char msg[CMOR_MAX_STRING];
 
     cmor_add_traceback("cmor_set_mapping_attribute");
 
@@ -497,10 +605,10 @@ void cmor_set_mapping_attribute(cmor_mappings_t * mapping,
 
     } else {
 
-        snprintf(msg, CMOR_MAX_STRING,
-                 "Unknown attribute: '%s' for mapping '%s' (value was: '%s')",
-                 att, mapping->id, val);
-        cmor_handle_error(msg, CMOR_WARNING);
+        cmor_handle_error_variadic(
+            "Unknown attribute: '%s' for mapping '%s' (value was: '%s')",
+            CMOR_WARNING,
+            att, mapping->id, val);
 
     }
     cmor_pop_traceback();
@@ -516,25 +624,24 @@ int cmor_set_grid_mapping(int gid, char *name, int nparam,
                           attributes_values[CMOR_MAX_GRID_ATTRIBUTES],
                           char *units, int lnunits)
 {
-    int grid_id, nattributes, ndims;
+    int grid_id, nattributes, ndims, msg_len;
     int i, j, k, l;
-    char *achar, *bchar;
+    char *achar, *bchar, *axes_msg;
     char lattributes_names[CMOR_MAX_GRID_ATTRIBUTES][CMOR_MAX_STRING];
     char lunits[CMOR_MAX_GRID_ATTRIBUTES][CMOR_MAX_STRING];
     char grid_attributes[CMOR_MAX_GRID_ATTRIBUTES][CMOR_MAX_STRING];
     char msg[CMOR_MAX_STRING];
-    char msg2[CMOR_MAX_STRING];
     char grid_dimensions[CMOR_MAX_DIMENSIONS][CMOR_MAX_STRING];
 
     cmor_add_traceback("cmor_set_grid_mapping");
     if (nparam >= CMOR_MAX_GRID_ATTRIBUTES) {
-        snprintf(msg, CMOR_MAX_STRING,
-                 "CMOR allows only %i grid parameters too be defined, "
-                 "you are trying to define %i parameters, if you really "
-                 "need that many recompile cmor changing the value of "
-                 "parameter: CMOR_MAX_GRID_ATTRIBUTES",
-                 CMOR_MAX_GRID_ATTRIBUTES, nparam);
-        cmor_handle_error(msg, CMOR_CRITICAL);
+        cmor_handle_error_variadic(
+            "CMOR allows only %i grid parameters too be defined, "
+            "you are trying to define %i parameters, if you really "
+            "need that many recompile cmor changing the value of "
+            "parameter: CMOR_MAX_GRID_ATTRIBUTES",
+            CMOR_CRITICAL,
+            CMOR_MAX_GRID_ATTRIBUTES, nparam);
     }
     achar = (char *)attributes_names;
     bchar = (char *)units;
@@ -554,11 +661,11 @@ int cmor_set_grid_mapping(int gid, char *name, int nparam,
                                             &ndims, grid_dimensions);
 
     if (ndims != cmor_grids[grid_id].ndims) {
-        snprintf(msg, CMOR_MAX_STRING,
-                 "you defined your grid with %i axes but grid_mapping "
-                 "'%s' requires exactly %i axes",
-                 cmor_grids[grid_id].ndims, name, ndims);
-        cmor_handle_error(msg, CMOR_CRITICAL);
+        cmor_handle_error_variadic(
+            "you defined your grid with %i axes but grid_mapping "
+            "'%s' requires exactly %i axes",
+            CMOR_CRITICAL,
+            cmor_grids[grid_id].ndims, name, ndims);
         cmor_pop_traceback();
         return (-1);
 
@@ -597,14 +704,22 @@ int cmor_set_grid_mapping(int gid, char *name, int nparam,
     }
 
     if (k != ndims) {
-        snprintf(msg, CMOR_MAX_STRING,
-                 "setting grid mapping to '%s' we could not find all "
-                 "the required axes, required axes are:", name);
+        msg_len = 0;
         for (i = 0; i < ndims; i++) {
-            snprintf(msg2, CMOR_MAX_STRING, " %s", grid_dimensions[i]);
-            strncat(msg, msg2, CMOR_MAX_STRING - strlen(msg));
+            msg_len += snprintf(NULL, 0, " %s", grid_dimensions[i]);
         }
-        cmor_handle_error(msg, CMOR_CRITICAL);
+        msg_len += 1;
+        axes_msg = (char *)malloc(msg_len * sizeof(char));
+        msg_len = 0;
+        for (i = 0; i < ndims; i++) {
+            msg_len += sprintf(&axes_msg[msg_len], " %s", grid_dimensions[i]);
+        }
+        cmor_handle_error_variadic(
+            "setting grid mapping to '%s' we could not find all "
+            "the required axes, required axes are:%s",
+            CMOR_CRITICAL,
+            name, axes_msg);
+        free(axes_msg);
         cmor_pop_traceback();
         return (-1);
 
@@ -625,10 +740,11 @@ int cmor_set_grid_mapping(int gid, char *name, int nparam,
                 cmor_set_grid_attribute(gid, lattributes_names[i],
                                         &attributes_values[i], lunits[i]);
             } else {
-                snprintf(msg, CMOR_MAX_STRING,
-                         "in grid_mapping, attribute '%s' (with value: %lf) is not a known attribute for grid mapping: '%s'",
-                         lattributes_names[i], attributes_values[i], name);
-                cmor_handle_error(msg, CMOR_WARNING);
+                cmor_handle_error_variadic(
+                    "in grid_mapping, attribute '%s' (with value: %lf) "
+                    "is not a known attribute for grid mapping: '%s'",
+                    CMOR_WARNING,
+                    lattributes_names[i], attributes_values[i], name);
                 cmor_pop_traceback();
                 return (-1);
             }
@@ -638,16 +754,17 @@ int cmor_set_grid_mapping(int gid, char *name, int nparam,
         }
     }
 /* -------------------------------------------------------------------- */
-/*      checks all parameter (but last 6 which are optional) have       */
+/*      checks all parameter (but last 7 which are optional) have       */
 /*      been set                                                        */
 /* -------------------------------------------------------------------- */
 
-    for (i = 0; i < nattributes - 6; i++) {
+    for (i = 0; i < nattributes - 7; i++) {
         if (cmor_has_grid_attribute(gid, grid_attributes[i]) == 1) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "Grid mapping attribute %s has not been set, you should consider setting it",
-                     grid_attributes[i]);
-            cmor_handle_error(msg, CMOR_WARNING);
+            cmor_handle_error_variadic(
+                "Grid mapping attribute %s has not been set, "
+                "you should consider setting it",
+                CMOR_WARNING,
+                grid_attributes[i]);
         }
     }
 
@@ -656,10 +773,60 @@ int cmor_set_grid_mapping(int gid, char *name, int nparam,
 /* -------------------------------------------------------------------- */
 
     strncpy(cmor_grids[grid_id].mapping, name, CMOR_MAX_STRING);
+    strncpy(cmor_grids[grid_id].name, name, CMOR_MAX_STRING);
     cmor_pop_traceback();
     return (0);
 }
 
+/************************************************************************/
+/*                       cmor_set_crs()                                 */
+/************************************************************************/
+
+int cmor_set_crs(int gid, char *grid_mapping, int nparam,
+    char *attributes_names, int lparams,
+    double attributes_values[CMOR_MAX_GRID_ATTRIBUTES],
+    char *units, int lnunits,
+    int ntextparam, char *text_attributes_names, int ltextparams,
+    char *text_attributes_values,
+    int text_attributes_length[CMOR_MAX_GRID_ATTRIBUTES])
+{
+    int i, ierr, grid_id, str_size;
+    char *achar, *bchar;
+    char ltext_attributes_name[CMOR_MAX_STRING];
+
+    cmor_add_traceback("cmor_set_grid_mapping");
+
+    ierr = cmor_set_grid_mapping(gid, grid_mapping, nparam,
+        attributes_names, lparams, attributes_values,
+        units, lnunits);
+
+    grid_id = -gid - CMOR_MAX_GRIDS;
+    strncpy(cmor_grids[grid_id].name, "crs", CMOR_MAX_STRING);
+
+    if (ntextparam >= CMOR_MAX_GRID_ATTRIBUTES) {
+        cmor_handle_error_variadic(
+            "CMOR allows only %i grid text parameters too be defined, "
+            "you are trying to define %i text parameters, if you really "
+            "need that many recompile cmor changing the value of "
+            "parameter: CMOR_MAX_GRID_ATTRIBUTES",
+            CMOR_CRITICAL,
+            CMOR_MAX_GRID_ATTRIBUTES, nparam);
+    }
+    achar = (char *)text_attributes_names;
+    bchar = (char *)text_attributes_values;
+    for (i = 0; i < ntextparam; i++) {
+        strncpy(ltext_attributes_name, achar, CMOR_MAX_STRING);
+        achar += ltextparams;
+        cmor_set_grid_text_attribute(gid, ltext_attributes_name,
+                                     bchar,
+                                     text_attributes_length[i]);
+        bchar += text_attributes_length[i];
+    }
+
+    cmor_pop_traceback();
+    return (ierr);
+}
+
 /************************************************************************/
 /*                 cmor_time_varying_grid_coordinate()                  */
 /************************************************************************/
@@ -686,6 +853,11 @@ int cmor_time_varying_grid_coordinate(int *coord_grid_id, int grid_id,
     cmor_add_traceback("cmor_time_varying_grid_coordinate");
     cmor_is_setup();
 
+    cmor_handle_error_variadic(
+        "DEPRECATED: cmor_time_varying_grid_coordinate is no longer supported "
+        "as of CMOR 3.11 and will be removed in a future release.",
+        CMOR_WARNING);
+
     strcpy(msg, "not found");
     if (coordinate_type == NULL) {
         for (j = 0; j < cmor_tables[table_id].nvars; j++) {
@@ -722,11 +894,12 @@ int cmor_time_varying_grid_coordinate(int *coord_grid_id, int grid_id,
           break;
       case (2):
           if (nvertices == 0) {
-              sprintf(msg,
-                      "your defining a vertices dependent variable (%s) "
-                      "associated with grid %i, but you declared this grid "
-                      "as having 0 vertices", table_entry, grid_id);
-              cmor_handle_error(msg, CMOR_CRITICAL);
+              cmor_handle_error_variadic(
+                "your defining a vertices dependent variable (%s) "
+                "associated with grid %i, but you declared this grid "
+                "as having 0 vertices",
+                CMOR_CRITICAL,
+                table_entry, grid_id);
           }
           if (cmor_grids[cmor_vars[*coord_grid_id].grid_id].associated_variables
               [3]
@@ -788,12 +961,14 @@ int cmor_time_varying_grid_coordinate(int *coord_grid_id, int grid_id,
       case (3):
           if (nvertices == 0) {
 
-              sprintf(msg, "your defining a vertices dependent "
-                      "variable (%s) associated with grid %i, "
-                      "but you declared this grid as having "
-                      "0 vertices", table_entry, grid_id);
+              cmor_handle_error_variadic(
+                "your defining a vertices dependent "
+                "variable (%s) associated with grid %i, "
+                "but you declared this grid as having "
+                "0 vertices",
+                CMOR_CRITICAL,
+                table_entry, grid_id);
 
-              cmor_handle_error(msg, CMOR_CRITICAL);
           }
 
           if (cmor_grids[cmor_vars[*coord_grid_id].grid_id].associated_variables
@@ -851,8 +1026,8 @@ int cmor_time_varying_grid_coordinate(int *coord_grid_id, int grid_id,
           break;
 
       default:
-          sprintf(msg, "unknown coord type: %i", ctype);
-          cmor_handle_error(msg, CMOR_CRITICAL);
+          cmor_handle_error_variadic(
+            "unknown coord type: %i", CMOR_CRITICAL, ctype);
           cmor_pop_traceback();
           return (-1);
 
@@ -878,18 +1053,18 @@ int cmor_grid(int *grid_id, int ndims, int *axes_ids, char type,
 
     cmor_add_traceback("cmor_grid");
     if ((axes_ids == NULL) || (ndims == 0)) {
-        snprintf(msg, CMOR_MAX_STRING,
-                 "You need to define the grid axes first");
-        cmor_handle_error(msg, CMOR_CRITICAL);
+        cmor_handle_error_variadic(
+            "You need to define the grid axes first", CMOR_CRITICAL);
     }
 
     cmor_ngrids += 1;
 
     if (cmor_ngrids >= CMOR_MAX_GRIDS) {
-        snprintf(msg, CMOR_MAX_STRING,
-                 "Too many grids defined, maximum possible "
-                 "grids is currently set to %i", CMOR_MAX_GRIDS);
-        cmor_handle_error(msg, CMOR_CRITICAL);
+        cmor_handle_error_variadic(
+            "Too many grids defined, maximum possible "
+            "grids is currently set to %i",
+            CMOR_CRITICAL,
+            CMOR_MAX_GRIDS);
     }
 
     n = 1;
@@ -897,9 +1072,10 @@ int cmor_grid(int *grid_id, int ndims, int *axes_ids, char type,
     for (i = 0; i < ndims; i++) {
 
         if (axes_ids[i] > cmor_naxes) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "Defining grid, Axis %i not defined yet", axes_ids[i]);
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "Defining grid, Axis %i not defined yet",
+                CMOR_CRITICAL,
+                axes_ids[i]);
         }
 
         if (cmor_tables[cmor_axes[axes_ids[i]].ref_table_id].axes
@@ -919,9 +1095,9 @@ int cmor_grid(int *grid_id, int ndims, int *axes_ids, char type,
     if (lat == NULL) {
 
         if (cmor_grids[cmor_ngrids].istimevarying != 1) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "you need to pass the latitude values when defining a grid");
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "you need to pass the latitude values when defining a grid",
+                CMOR_CRITICAL);
         }
 
     } else {
@@ -939,10 +1115,9 @@ int cmor_grid(int *grid_id, int ndims, int *axes_ids, char type,
     if (lon == NULL) {
 
         if (cmor_grids[cmor_ngrids].istimevarying != 1) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "you need to pass the longitude values when "
-                     "defining a grid");
-            cmor_handle_error(msg, CMOR_CRITICAL);
+            cmor_handle_error_variadic(
+                "you need to pass the longitude values when "
+                "defining a grid", CMOR_CRITICAL);
         }
 
     } else {
@@ -957,10 +1132,9 @@ int cmor_grid(int *grid_id, int ndims, int *axes_ids, char type,
     }
     if (blat == NULL) {
         if (cmor_grids[cmor_ngrids].istimevarying != 1) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "it is recommended you pass the latitude bounds "
-                     "values when defining a grid");
-            cmor_handle_error(msg, CMOR_WARNING);
+            cmor_handle_error_variadic(
+                "it is recommended you pass the latitude bounds "
+                "values when defining a grid", CMOR_WARNING);
         }
     } else {
         cmor_copy_data(&cmor_grids[cmor_ngrids].blats, blat, type,
@@ -1009,9 +1183,9 @@ int cmor_grid(int *grid_id, int ndims, int *axes_ids, char type,
     if (blon == NULL) {
 
         if (cmor_grids[cmor_ngrids].istimevarying != 1) {
-            snprintf(msg, CMOR_MAX_STRING,
-                     "it is recommended you pass the longitude bounds values when defining a grid");
-            cmor_handle_error(msg, CMOR_WARNING);
+            cmor_handle_error_variadic(
+                "it is recommended you pass the longitude bounds values when defining a grid",
+                CMOR_WARNING);
         }
 
     } else {
