@@ -2,7 +2,7 @@
 #define CMOR_H
 
 #define CMOR_VERSION_MAJOR 3
-#define CMOR_VERSION_MINOR 9
+#define CMOR_VERSION_MINOR 11
 #define CMOR_VERSION_PATCH 0
 
 #define CMOR_CF_VERSION_MAJOR 1
@@ -48,6 +48,10 @@
 #define CMOR_APPEND CMOR_APPEND_4
 #define CMOR_REPLACE CMOR_REPLACE_4
 
+#define CMOR_APPROX_INTERVAL_DEFAULT          0.
+#define CMOR_APPROX_INTERVAL_WARNING_DEFAULT  .1
+#define CMOR_APPROX_INTERVAL_ERROR_DEFAULT    .2
+
 #define CMOR_INPUTFILENAME       GLOBAL_INTERNAL"dataset_json"
 #define CV_INPUTFILENAME         GLOBAL_INTERNAL"CV"
 #define CV_CHECK_ERROR           GLOBAL_INTERNAL"CV_ERROR"
@@ -125,6 +129,9 @@
 #define VARIABLE_ATT_EXTCELLMEASURES  "ext_cell_measures"
 #define VARIABLE_ATT_CELLMEASURES     "cell_measures"
 #define VARIALBE_ATT_GRIDMAPPING      "grid_mapping"
+#define VARIALBE_ATT_QUANTIZATION     "quantization"
+#define VARIALBE_ATT_QUANTIZATION_NSB "quantization_nsb"
+#define VARIALBE_ATT_QUANTIZATION_NSD "quantization_nsd"
 
 #define VARIABLE_ATT_VALIDMIN         "valid_min"
 #define VARIABLE_ATT_VALIDMAX         "valid_max"
@@ -135,13 +142,18 @@
 #define VARIABLE_ATT_DEFLATE          "deflate"
 #define VARIABLE_ATT_DEFLATELEVEL     "deflate_level"
 #define VARIABLE_ATT_ZSTANDARDLEVEL   "zstandard_level"
-#define VARIABLE_ATT_QUANTIZEMODE     "quantize_mode"
-#define VARIABLE_ATT_QUANTIZENSD      "quantize_nsd"
 #define VARIABLE_ATT_MODELINGREALM    "modeling_realm"
 #define VARIALBE_ATT_FREQUENCY        "frequency"
 #define VARIABLE_ATT_FLAGVALUES       "flag_values"
 #define VARIABLE_ATT_FLAGMEANINGS     "flag_meanings"
 #define VARIABLE_ATT_OUTNAME          "out_name"
+#define VARIABLE_ATT_BRANDDESCRIPTION "brand_description"
+#define VARIABLE_ATT_BRANDINGSUFFIX   "branding_suffix"
+#define VARIABLE_ATT_TEMPORALLABEL    "temporal_label"
+#define VARIABLE_ATT_VERTICALLABEL    "vertical_label"
+#define VARIABLE_ATT_HORIZONTALLABEL  "horizontal_label"
+#define VARIABLE_ATT_AREALABEL        "area_label"
+#define VARIABLE_ATT_VARIABLE_TITLE   "variable_title"
 #define COMMENT_VARIABLE_ZFACTOR      "use formula table"
 #define GLOBAL_SEPARATORS             "><"
 #define GLOBAL_OPENOPTIONAL           "["
@@ -160,6 +172,11 @@
 #define GLOBAL_ATT_ACTIVITY_ID        "activity_id"
 #define GLOBAL_ATT_VAL_NODRIVER       "no-driver"
 #define GLOBAL_ATT_VARIABLE_ID        "variable_id"
+#define GLOBAL_ATT_BRANDINGSUFFIX     "branding_suffix"
+#define GLOBAL_ATT_TEMPORALLABEL      "temporal_label"
+#define GLOBAL_ATT_VERTICALLABEL      "vertical_label"
+#define GLOBAL_ATT_HORIZONTALLABEL    "horizontal_label"
+#define GLOBAL_ATT_AREALABEL          "area_label"
 #define GLOBAL_ATT_SOURCE_ID          "source_id"
 #define GLOBAL_ATT_SOURCE             "source"
 #define GLOBAL_ATT_SUB_EXPT_ID        "sub_experiment_id"
@@ -182,6 +199,9 @@
 #define GLOBAL_ATT_DATASPECSVERSION   "data_specs_version"
 #define GLOBAL_ATT_FREQUENCY          "frequency"
 #define GLOBAL_ATT_LICENSE            "license"
+#define GLOBAL_ATT_LICENSE_ID         "license_id"
+#define GLOBAL_ATT_LICENSE_TYPE       "license_type"
+#define GLOBAL_ATT_LICENSE_URL        "license_url"
 #define GLOBAL_ATT_TRACKING_PREFIX    "tracking_prefix"
 #define GLOBAL_ATT_CALENDAR           "calendar"
 #define GLOBAL_ATT_INSTITUTION_ID     "institution_id"
@@ -195,6 +215,7 @@
 #define GLOBAL_ATT_MIP_ERA            "mip_era"
 #define GLOBAL_CV_FILENAME            GLOBAL_INTERNAL"controlled_vocabulary_file"
 #define GLOBAL_IS_CMIP6               GLOBAL_INTERNAL"cmip6_option"
+#define GLOBAL_IS_CMIP7               GLOBAL_INTERNAL"cmip7_option"
 
 #define NO_PARENT                     "no parent"
 #define NONE                          "none"
@@ -217,6 +238,7 @@
 #define JSON_KEY_CV_ENTRY             "CV"
 
 #define CV_KEY_REQUIRED_GBL_ATTRS     "required_global_attributes"
+#define CV_KEY_FREQUENCY              "frequency"
 #define CV_KEY_INSTITUTION_ID         "institution_id"
 #define CV_KEY_EXPERIMENT_ID          "experiment_id"
 #define CV_KEY_SOURCE_IDS             "source_id"
@@ -225,6 +247,17 @@
 #define CV_KEY_GRIDLABEL_GR           "gr"
 #define CV_KEY_SOURCE_LABEL           "source"
 #define CV_KEY_SUB_EXPERIMENT_ID      "sub_experiment_id"
+#define CV_KEY_BRANDING_TEMPLATE      "branding_suffix"
+#define CV_KEY_APRX_INTRVL            "approx_interval"
+#define CV_KEY_APRX_INTRVL_ERR        "approx_interval_error"
+#define CV_KEY_APRX_INTRVL_WRN        "approx_interval_warning"
+#define CV_KEY_DATASPECSVERSION       "data_specs_version"
+#define CV_KEY_MIP_ERA                "mip_era"
+#define CV_KEY_LICENSE                "license"
+#define CV_KEY_LICENSE_ID             "license_id"
+#define CV_KEY_LICENSE_TEMPLATE       "license_template"
+#define CV_KEY_LICENSE_TYPE           "license_type"
+#define CV_KEY_LICENSE_URL            "license_url"
 
 #define CV_EXP_ATTR_ADDSOURCETYPE     "additional_allowed_model_components"
 #define CV_EXP_ATTR_REQSOURCETYPE     "required_model_components"
@@ -251,6 +284,13 @@
 #define TABLE_HEADER_INT_MISSING_VALUE    "int_missing_value"
 #define TABLE_HEADER_MAGIC_NUMBER     "magic_number"
 #define TABLE_HEADER_DATASPECSVERSION "data_specs_version"
+#define TABLE_HEADER_VALIDMIN         "valid_min"
+#define TABLE_HEADER_VALIDMAX         "valid_max"
+#define TABLE_HEADER_MINMEANABS       "ok_min_mean_abs"
+#define TABLE_HEADER_MAXMEANABS       "ok_max_mean_abs"
+#define TABLE_HEADER_POSITIVE         "positive"
+#define TABLE_HEADER_CHECKSUM         "checksum"
+#define TABLE_HEADER_TYPE             "type"
 #define OUTPUT_TEMPLATE_RIPF          "run_variant"
 
 #define DIMENSION_LATITUDE            "latitude"
@@ -264,6 +304,7 @@
 #define AREA                          "area"
 #define VOLUME                        "volume"
 #define CMIP6                         "CMIP6"
+#define CMIP7                         "CMIP7"
 #define CMOR_DEFAULT_PATH_TEMPLATE    "<mip_era><activity_id><institution_id><source_id><experiment_id><member_id><table><variable_id><grid_label><version>"
 #define CMOR_DEFAULT_FILE_TEMPLATE    "<variable_id><table><source_id><experiment_id><member_id><grid_label>"
 #define CMOR_DEFAULT_FURTHERURL_TEMPLATE "https://furtherinfo.es-doc.org/<mip_era><institution_id><source_id><experiment_id><sub_experiment_id><variant_label>"
@@ -271,6 +312,14 @@
 //#define EXTERNAL_VARIABLE_REGEX       "([[:alpha:]]+):[[:blank:]]*([[:alpha:]]+)[[:blank:]]*([[:alpha:]]+:[[:blank:]]*([[:alpha:]]+))*"
 #define EXTERNAL_VARIABLE_REGEX       "[[:alpha:]]+:[[:blank:]]*([[:alpha:]]+)([[:blank:]]*[[:alpha:]]+:[[:blank:]]*([[:alpha:]]+))*"
 
+#define QUANTIZATION_INFO              "quantization_info"
+#define QUANTIZATION_ALGORITHM         "algorithm"
+#define QUANTIZATION_IMPLEMENTATION    "implementation"
+#define QUANTIZATION_NOQUANTIZATION    "no_quantization"
+#define QUANTIZATION_BITGROOM          "bitgroom"
+#define QUANTIZATION_BITROUND          "bitround"
+#define QUANTIZATION_GRANULARBITROUND  "granular_bitround"
+
 extern int CMOR_TERMINATE_SIGNAL;
 extern int USE_NETCDF_4;
 extern int CMOR_MODE;
@@ -295,6 +344,7 @@ extern char cmor_traceback_info[CMOR_MAX_STRING];
 
 typedef struct cmor_grid_ {
     int id;
+    char name[CMOR_MAX_STRING];
     char mapping[CMOR_MAX_STRING];
     int nattributes;
     char attributes_names[CMOR_MAX_GRID_ATTRIBUTES][CMOR_MAX_STRING];
@@ -312,6 +362,9 @@ typedef struct cmor_grid_ {
 /*      for lon/lat/blon/blat/area/volumes                              */
 /* -------------------------------------------------------------------- */
     int associated_variables[6];
+    int ntextattributes;
+    char text_attributes_names[CMOR_MAX_GRID_ATTRIBUTES][CMOR_MAX_STRING];
+    char *text_attributes_values[CMOR_MAX_GRID_ATTRIBUTES];
 } cmor_grid_t;
 
 extern cmor_grid_t cmor_grids[CMOR_MAX_GRIDS];
@@ -398,12 +451,17 @@ typedef struct cmor_axis_ {
     int hybrid_in;
     int hybrid_out;
     int store_in_netcdf;
+    double approx_interval;
+    double approx_interval_warning;
+    double approx_interval_error;
 } cmor_axis_t;
 extern cmor_axis_t cmor_axes[CMOR_MAX_AXES];
 
 typedef struct cmor_variable_def_ {
     int table_id;
     char id[CMOR_MAX_STRING];
+    char brand_description[CMOR_MAX_STRING];
+    char variable_title[CMOR_MAX_STRING];
     char standard_name[CMOR_MAX_STRING];
     char units[CMOR_MAX_STRING];
     char cell_methods[CMOR_MAX_STRING];
@@ -426,13 +484,16 @@ typedef struct cmor_variable_def_ {
     int deflate;
     int deflate_level;
     int zstandard_level;
-    int quantize_mode;
-    int quantize_nsd;
     char required[CMOR_MAX_STRING];
     char realm[CMOR_MAX_STRING];
     char frequency[CMOR_MAX_STRING];
     char out_name[CMOR_MAX_STRING];
     char generic_level_name[CMOR_MAX_STRING];
+    char branding_suffix[CMOR_MAX_STRING];
+    char temporal_label[CMOR_MAX_STRING];
+    char vertical_label[CMOR_MAX_STRING];
+    char horizontal_label[CMOR_MAX_STRING];
+    char area_label[CMOR_MAX_STRING];
 } cmor_var_def_t;
 
 typedef struct cmor_var_ {
@@ -498,6 +559,11 @@ typedef struct cmor_var_ {
     char suffix[CMOR_MAX_STRING];
     int suffix_has_date;
     char frequency[CMOR_MAX_STRING];
+    char branding_suffix[CMOR_MAX_STRING];
+    char temporal_label[CMOR_MAX_STRING];
+    char vertical_label[CMOR_MAX_STRING];
+    char horizontal_label[CMOR_MAX_STRING];
+    char area_label[CMOR_MAX_STRING];
 } cmor_var_t;
 
 extern cmor_var_t cmor_vars[CMOR_MAX_VARIABLES];
@@ -524,8 +590,9 @@ typedef struct cmor_table_ {
     int nexps;
     int nmappings;
     float cf_version;
-    float cmor_version;
+    char cmor_version[CMOR_MAX_STRING];
     char mip_era[CMOR_MAX_STRING];
+    char checksum[CMOR_MAX_STRING];
     char Conventions[CMOR_MAX_STRING];
     char data_specs_version[CMOR_MAX_STRING];
     char szTable_id[CMOR_MAX_STRING];
@@ -537,6 +604,12 @@ typedef struct cmor_table_ {
     cmor_var_def_t formula[CMOR_MAX_FORMULA];
     cmor_mappings_t mappings[CMOR_MAX_ELEMENTS];
     cmor_CV_def_t *CV;
+    char positive;
+    char type;
+    float valid_min;
+    float valid_max;
+    float ok_min_mean_abs;
+    float ok_max_mean_abs;
     double missing_value;
     long    int_missing_value;
     double interval;
