@@ -8,6 +8,7 @@
 #include <udunits2.h>
 #include "cdmsint.h"
 #include <stdio.h>
+#include <stdarg.h>
 #include <json-c/json.h>
 
 extern void cmor_md5( FILE * inputfile, unsigned char checksum[16] );
@@ -26,10 +27,13 @@ extern int cmor_have_NetCDF4( void );
 extern int cmor_have_NetCDF41min( void );
 extern int cmor_have_NetCDF3( void );
 extern int cmor_have_NetCDF363( void );
-extern void cmor_handle_error( char error_msg[CMOR_MAX_STRING],
-			       int level );
-extern void cmor_handle_error_var( char error_msg[CMOR_MAX_STRING], int level,
-                                   int var_id );
+extern void cmor_handle_error_internal( char *error_msg, int level );
+extern void cmor_handle_error( char *error_msg, int level );
+extern void cmor_handle_error_variadic( char *error_msg, int level, ... )
+    __attribute__ ((__format__(printf, 1, 3)));
+extern void cmor_handle_error_var( char *error_msg, int level, int var_id );
+extern void cmor_handle_error_var_variadic( char *error_msg, int level, int var_id, ... )
+    __attribute__ ((__format__(printf, 1, 4)));
 extern int cmor_setup( char *path, int *netcdf, int *verbosity, int *mode,
 		       char *logfile, int *cmor_create_subdirectories);
 
@@ -112,7 +116,7 @@ extern void create_singleton_dimensions(int var_id, int ncid,
 extern int copyfile(const char *source, const char *dest);
 
 /* ==================================================================== */
-/*      Control Vocabulary                                              */
+/*      Controlled Vocabulary                                              */
 /* ==================================================================== */
 
 extern int cmor_build_outname(int var_id, char *outname );
@@ -144,10 +148,12 @@ extern cmor_CV_def_t * cmor_CV_rootsearch(cmor_CV_def_t *CV, char *key);
 extern int cmor_CV_checkFurtherInfoURL(int var_id);
 extern int cmor_CV_checkGrids(cmor_CV_def_t *CV);
 extern int cmor_CV_setInstitution( cmor_CV_def_t *CV);
+extern int cmor_CV_setLicense( cmor_CV_def_t *CV);
 
 extern int cmor_CV_set_entry(cmor_table_t* table, json_object *value);
 extern int  cmor_CV_ValidateGblAttributes( char *name);
 extern int cmor_CV_ValidateAttribute(cmor_CV_def_t *CV, char *szValue);
+extern int cmor_CV_check_branding_suffix(cmor_CV_def_t *CV);
 
 
 
@@ -156,9 +162,8 @@ extern int cmor_CV_ValidateAttribute(cmor_CV_def_t *CV, char *szValue);
 /* ==================================================================== */
 
 extern void cmor_init_axis_def( cmor_axis_def_t * axis, int table_id );
-extern int cmor_set_axis_def_att( cmor_axis_def_t * axis,
-				  char att[CMOR_MAX_STRING],
-				  char val[CMOR_MAX_STRING] );
+extern int cmor_set_axis_def_att( cmor_axis_def_t * axis, char *att,
+                                    char *val );
 extern void cmor_trim_string( char *in, char *out );
 extern int cmor_calendar_c2i( char *calendar, cdCalenType * ical );
 extern int cmor_convert_time_units( char *inunits, char *outunits,
@@ -228,9 +233,8 @@ extern int cmor_set_quantize( int var_id, int quantize_mode,
 extern int cmor_set_chunking( int var_id, int nTableID,
 							    size_t nc_dim_chunking[]);
 
-extern int cmor_set_var_def_att( cmor_var_def_t * var,
-				 char att[CMOR_MAX_STRING],
-				 char val[CMOR_MAX_STRING] );
+extern int cmor_set_var_def_att( cmor_var_def_t * var, char *att,
+                                    char *val);
 extern int cmor_get_variable_time_length( int *var_id, int *length );
 extern int cmor_get_original_shape( int *var_id, int *shape_array,
 				    int *rank, int blank_time );
@@ -253,6 +257,9 @@ extern int cmor_get_grid_attribute( int gid, char *name, double *value );
 extern void cmor_convert_value( char *units, char *ctmp, double *tmp );
 extern int cmor_set_grid_attribute( int gid, char *name, double *value,
 				    char *units );
+extern int cmor_has_grid_text_attribute( int gid, char *name );
+extern int cmor_get_grid_text_attribute( int gid, char *name, char *value );
+extern int cmor_set_grid_text_attribute( int gid, char *name, char *value, int size );
 extern int cmor_attribute_in_list( char *name, int n,
 				   char ( *atts )[CMOR_MAX_STRING] );
 extern int cmor_grid_valid_mapping_attribute_names( char *name, int *natt,
@@ -266,6 +273,13 @@ extern int cmor_set_grid_mapping( int gid, char *name, int nparam,
 				  double attributes_values[CMOR_MAX_GRID_ATTRIBUTES],
 				  char *units,
 				  int lnunits );
+extern int cmor_set_crs( int gid, char *grid_mapping, int nparam,
+                    char *attributes_names, int lparams,
+                    double attributes_values[CMOR_MAX_GRID_ATTRIBUTES],
+                    char *units, int lnunits,
+                    int ntextparam, char *text_attributes_names, int ltextparams,
+                    char *text_attributes_values,
+                    int text_attributes_length[CMOR_MAX_GRID_ATTRIBUTES]);
 extern int cmor_grid( int *grid_id, int ndims, int *axes_ids, char type,
 		      void *lat, void *lon, int nvertices, void *blat,
 		      void *blon );
